'''
Process and save molecular fingerprints from SMILES strings.
Saves encoded substructures in SMILES and SMARTS format for
later substructure matching and fingerprint explanation.

author: Rasmus Kronberg
email: rasmus.kronberg@aalto.fi
'''

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Draw
import numpy as np
import pandas as pd


class InputError(Exception):
    pass


class Fingerprint:
    def __init__(self, data, data_path):

        # Initialize
        if isinstance(data, pd.DataFrame) and 'SMILES' in data.columns:
            self.data = data
            self.data_path = data_path
        else:
            raise InputError('pandas.DataFrame with SMILES column expected')

    def ecfp(self, nbits=64, diameter=4):

        # Generate circular ECFP fingerprints from SMILES and save bitinfo
        self.fingerprints = np.zeros((self.data.shape[0], nbits))

        smiles = self.data['SMILES'].to_numpy()
        size = len(smiles)
        print(' IN ECFP len(smi) ',size)
        for i in range(size):
           mol = Chem.MolFromSmiles(smiles[i])
           print(i,'smiles ',smiles[i],mol)
           name='all-mol/redo-mol%d.png'%i
           Draw.MolToFile(mol,name)  # draw all molecules

#        print(' IN ECFP  all smi ',smiles)
        submol_smiles = [[] for _ in range(nbits)]
        submol_smarts = [[] for _ in range(nbits)]

        for i, smi in enumerate(smiles):
            print('Processing... %.1f %%' % (100*i/size), end='\r')
            bitinfo = {}
            mol = Chem.MolFromSmiles(smi)
            fp = AllChem.GetMorganFingerprintAsBitVect(
                mol, int(0.5*diameter), nBits=nbits, bitInfo=bitinfo, useChirality=True)
            self.fingerprints[i] = np.array(fp)

            # Construct molecular fragments from bitinfo
            for bit in bitinfo:
                (atom, radius) = bitinfo[bit][0]
                if radius > 0:
                    env = Chem.FindAtomEnvironmentOfRadiusN(mol, radius, atom)
                    atoms = []
                    for b in env:
                        atoms.append(mol.GetBondWithIdx(b).GetBeginAtomIdx())
                        atoms.append(mol.GetBondWithIdx(b).GetEndAtomIdx())
                    atoms = list(set(atoms))
                else:
                    atoms = [atom]
                    env = None

                # Write detailed info to SMARTS (ring info, coord number...)
                symbols = []
                for at in mol.GetAtoms():
                    nHs = at.GetTotalNumHs()
                    sym = '[' + at.GetSmarts().strip('[]')
                    if nHs:
                        sym += 'H'
                        if nHs > 1:
                            sym += '%d' % nHs
                    if at.IsInRing():
                        sym += ';R'
                    else:
                        sym += ';!R'
                    sym += ';D%d' % at.GetDegree()
                    sym += ']'
                    symbols.append(sym)

                frag2smiles = Chem.MolFragmentToSmiles(
                    mol, atoms, bondsToUse=env, allHsExplicit=True,
                    allBondsExplicit=True, rootedAtAtom=atom,
                    isomericSmiles=False)
                frag2smarts = Chem.MolFragmentToSmiles(
                    mol, atoms, bondsToUse=env, atomSymbols=symbols,
                    allBondsExplicit=True, rootedAtAtom=atom,
                    isomericSmiles=False)

                submol_smiles[bit].append(frag2smiles)
                submol_smarts[bit].append(frag2smarts)

        # As bit collisions are possible, store most
        # frequently encoded submol for each bit
        self.subsmiles = ['NaN']*nbits
        self.subsmarts = ['NaN']*nbits

        for i in range(nbits):
            unique_subsmiles, pos_subsmiles = np.unique(np.array(
                submol_smiles[i]), return_inverse=True)
            unique_subsmarts, pos_subsmarts = np.unique(np.array(
                submol_smarts[i]), return_inverse=True)
            cnt_subsmiles = np.bincount(pos_subsmiles)
            cnt_subsmarts = np.bincount(pos_subsmarts)
            if cnt_subsmiles.size:
                self.subsmiles[i] = unique_subsmiles[np.argmax(cnt_subsmiles)]
                self.subsmarts[i] = unique_subsmarts[np.argmax(cnt_subsmarts)]

        pd.DataFrame(
            {'Bit': np.arange(nbits),
             'SMILES': self.subsmiles,
             'SMARTS': self.subsmarts}
            ).to_csv('%s/submolecules.csv' % self.data_path, index=False)
        pd.DataFrame(
            self.fingerprints
            ).to_csv('%s/fingerprints.csv' % self.data_path, index=False)
