import os
import shutil
import numpy as np
from glob import glob

def write_squscheme(outfile):
  outfile.write(
     """
     # # # # # # # # # # # # # # # # # # # # # # # # # #
     #  (0)                 (1)                  (2)   # 
     # [AH2] ---E01--->   [AH21+]  ---E12--->  [AH22+] #
     #  .   .                .   .                .    #
     #  |      .             |      .             |    #
     # pKa03   PCET04      pKa14    PCET15     pKa25   #  
     #  |           .        |            .       |    #
     #  .              .     .               .    .    #
     # (3)               .  (4)                . (5)   # 
     # [AH-]   ---E34--->  [AH]    ---E45--->   [AH+]  #
     #  .   .                .  .                 .    #
     #  |      .             |    .               |    #
     # pKa36   PCET37      pKa47    PCET48      pKa58  #
     #  |          .         |          .         |    #
     #  .             .      .             .      .    #
     # (6)               .  (7)               .  (8)   #
     # [A2-]   ---E67--->  [A1-]    ---E78--->   [A]   #
     # # # # # # # # # # # # # # # # # # # # # # # # # #
     """
     )

# finds the total energy out of Gaussian output.
def get_energy(infile):
  scfEne = []
  with open(infile) as gfile:
    line = gfile.readline()
    while line != "":
      if "SCF Done:" in line:
        ll = line.split()
        scfEne.append(float(ll[4]))
      line = gfile.readline()
  return scfEne[-1]

# finds the thermal corection term out of Gaussian output.
def get_Gcorr(infile):
  Ene = []
  with open(infile) as gfile:
    line = gfile.readline()
    while line != "":
      if "Thermal correction to Gibbs Free Energy=" in line:
        ll = line.split()
        Ene.append(float(ll[-1]))
      line = gfile.readline()
  return Ene[-1]

#computes Gibbs free energy for proton transfer step
def compute_GPT(GFEs, Gproton):
    gfes = np.loadtxt(GFEs)
    gPT03 = float(gfes[3]) + Gproton - float(gfes[0])
    gPT14 = float(gfes[4]) + Gproton - float(gfes[1])
    gPT25 = float(gfes[5]) + Gproton - float(gfes[2])
    gPT36 = float(gfes[6]) + Gproton - float(gfes[3])
    gPT47 = float(gfes[7]) + Gproton - float(gfes[4])
    gPT58 = float(gfes[8]) + Gproton - float(gfes[5])

    return gPT03, gPT14, gPT25, gPT36, gPT47, gPT58

#computes Gibbs free energy for electron-proton transfer step
def compute_GPCET(GFEs, GH2):
    gfes = np.loadtxt(GFEs)
    gPCET04 = float(gfes[4]) + (0.5*GH2) - float(gfes[0])
    gPCET15 = float(gfes[5]) + (0.5*GH2) - float(gfes[1])
    gPCET37 = float(gfes[7]) + (0.5*GH2) - float(gfes[3])
    gPCET48 = float(gfes[8]) + (0.5*GH2) - float(gfes[4])

    return gPCET04, gPCET15, gPCET37, gPCET48 

#computes pKa of proton transfer step
def compute_pka(GPT):
    pka = []
    for g in GPT:
      pka.append(g/0.0595)

    return pka

#scales pKa regarding the scalig relation for M06-2X xc-functional.
def pka_isscaled(pka):
    pkascaled = []
    for pKai in pka:
       pkascaled.append(pKai * 0.49 + 3.2)
    return pkascaled

#computes scaled value of Gibbs free energy for PT step.
def GPT_isscalled(pka_sclaed):
    GPT_scalled = []
    for pka_scaledi in pka_sclaed:
       GPT_scalled.append(0.0595*pka_scaledi)
    return GPT_scalled

#computes scaled value of Gibbs free energy for ET step.
def compute_GET(GPT_scalled, GPCET):
    gET01 = GPCET[0] - GPT_scalled[1]
    gET12 = GPCET[1] - GPT_scalled[2]
    gET34 = GPCET[0] - GPT_scalled[0]
    gET45 = GPCET[1] - GPT_scalled[1]
    gET67 = GPCET[2] - GPT_scalled[3]
    gET78 = GPCET[3] - GPT_scalled[4]
    return gET01, gET12, gET34, gET45, gET67, gET78


#freeEnergy.dat is written for each molecule!
heading = "#G_{AH2}(eV) G_{AH21+}(eV) G_{AH22+}(eV) G_{AH1-}(eV) G_{AH}(eV) G_{AH1+}(eV) G{A2-}(eV) G_{A1-}(eV) G_{A}(eV)\n"

efile = open("freeEnergy.dat", "w")
efile.write(heading)
gfe = []
for folder in ['AH2', 'AH21+', 'AH22+', 'AH1-', 'AH', 'AH1+', 'A2-', 'A1-', 'A']:
  pm7file = "{}/input.log".format(folder)
  smdfile = "{}/inputsmd.log".format(folder)
  E0 = get_energy(smdfile)
  Gcorr = get_Gcorr(pm7file)
  GFE = E0 + Gcorr
  GFEtoeV = GFE*27.211
  gfe.append(GFEtoeV)

efile.write("{:.3f}  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:.3f}  {:.3f}".format(gfe[0],gfe[1],gfe[2],gfe[3],gfe[4],gfe[5],gfe[6],gfe[7],gfe[8]))
efile.close()

GH2 = -31.75624
Gproton = -11.861
Gelect = -4.01617

GPT= compute_GPT("freeEnergy.dat", Gproton)
GPCET = compute_GPCET("freeEnergy.dat", GH2)
pka = compute_pka(GPT)
pka_scaled = pka_isscaled(pka)
GPT_scaled = GPT_isscalled(pka_scaled)
GET_scaled = compute_GET(GPT_scaled, GPCET)


outfile = open("schemesquare.dat", "w")
write_squscheme(outfile)

#pKa is written:
outfile.write("\n")
outfile.write("pKa of PTs:\n")

pkaList = ["pKa03", "pKa14", "pKa25", "pKa36", "pKa47", "pKa58"]
for a, pka in enumerate(pka_scaled):
  outfile.write("{} = {:3.3f}\n".format(pkaList[a], pka))

#redox potential is written:
outfile.write("\n")
outfile.write("Redox potential of ETs in V:\n")

redoxList = ["E01", "E12", "E34", "E45", "E67", "E78"]
for e, get_scaled in enumerate(GET_scaled):
  outfile.write("{} = {:3.3f}\n".format(redoxList[e], -get_scaled))


outfile.write("\n")
outfile.write(" -----------------------\n")
outfile.write(" |SOME MORE INFORMATION|\n")
outfile.write(" -----------------------\n")

#Proton transfer Gibbs free energy
outfile.write("\n")
outfile.write("Gibbs free energies of PTs in eV:\n")

gptList = ["GPT03", "GPT14", "GPT25", "GPT36", "GPT47", "GPT58"]
for i, gpt in enumerate(GPT_scaled):
  outfile.write("{} = {:3.3f}\n".format(gptList[i], gpt))

#Proton-electron transfer Gibbs free energy
outfile.write("\n")
outfile.write("Gibbs free energies of PCETs in eV:\n")

gpcetList = ["GPCET04", "GPCET15", "GPCET37", "GPCET48"]
for j, gpcet  in enumerate(GPCET):
  outfile.write("{} = {:3.3f}\n".format(gpcetList[j], gpcet))

#Electron transfer Gibbs free energy
outfile.write("\n")
outfile.write("Gibbs free energies of ETs in eV:\n")

getList = ["GET01", "GET12", "GET34", "GET45", "GET67", "GET78"]
for k, get in enumerate(GET_scaled):
  outfile.write("{} = {:3.3f}\n".format(getList[k], get))

outfile.close()













